@@ -23,10 +23,10 @@ impl Audit for UnredactedSecrets {
         Ok(Self)
     }
 
-    fn audit_raw<'w>(
+    fn audit_raw<'doc>(
         &self,
-        input: &'w super::AuditInput,
-    ) -> anyhow::Result<Vec<crate::finding::Finding<'w>>> {
+        input: &'doc super::AuditInput,
+    ) -> anyhow::Result<Vec<crate::finding::Finding<'doc>>> {
         let mut findings = vec![];
 
         for (expr, span) in parse_expressions_from_input(input) {
@@ -35,8 +35,6 @@ impl Audit for UnredactedSecrets {
                 continue;
             };
 
-            expr.as_curly();
-
             for _ in Self::secret_leakages(&parsed) {
                 findings.push(
                     Self::finding()
@@ -81,7 +79,7 @@ impl UnredactedSecrets {
                 }
             }
             Expr::Index(expr) => results.extend(Self::secret_leakages(expr)),
-            Expr::Context(Context { raw: _, components }) => {
+            Expr::Context(Context { components, .. }) => {
                 results.extend(components.iter().flat_map(Self::secret_leakages))
             }
             Expr::BinOp { lhs, op: _, rhs } => {
