@@ -9,7 +9,7 @@ use anstream::{eprintln, stream::IsTerminal};
 use anyhow::{Context, Result, anyhow};
 use audit::{Audit, AuditLoadError};
 use camino::{Utf8Path, Utf8PathBuf};
-use clap::{Parser, ValueEnum};
+use clap::{CommandFactory, Parser, ValueEnum};
 use clap_verbosity_flag::InfoLevel;
 use config::Config;
 use finding::{Confidence, Persona, Severity};
@@ -17,9 +17,8 @@ use github_actions_models::common::Uses;
 use github_api::GitHubHost;
 use ignore::WalkBuilder;
 use indicatif::ProgressStyle;
-use models::Action;
 use owo_colors::OwoColorize;
-use registry::{AuditRegistry, FindingRegistry, InputRegistry};
+use registry::{AuditRegistry, FindingRegistry, InputKey, InputKind, InputRegistry};
 use state::AuditState;
 use tracing::{Span, info_span, instrument};
 use tracing_indicatif::{IndicatifLayer, span_ext::IndicatifSpanExt};
@@ -116,10 +115,20 @@ struct App {
 
     /// Control which kinds of inputs are collected for auditing.
     ///
-    /// By default, all workflows and composite actions are collected.
+    /// By default, all workflows and composite actions are collected,
+    /// while honoring `.gitignore` files.
     #[arg(long, value_enum, default_value_t)]
     collect: CollectionMode,
 
+    /// Fail instead of warning on syntax and schema errors
+    /// in collected inputs.
+    #[arg(long)]
+    strict_collection: bool,
+
+    /// Generate tab completion scripts for the specified shell.
+    #[arg(long, value_enum, value_name = "SHELL", exclusive = true)]
+    completions: Option<clap_complete::Shell>,
+
     /// Enable naches mode.
     #[arg(long, hide = true, env = "ZIZMOR_NACHES")]
     naches: bool,
@@ -281,17 +290,18 @@ fn collect_from_dir(
                 .parent()
                 .is_some_and(|dir| dir.ends_with(".github/workflows"))
         {
-            registry
-                .register_by_path(entry, Some(input_path))
-                .with_context(|| format!("failed to register input: {entry}"))?;
+            let key = InputKey::local(entry, Some(input_path))?;
+            let contents = std::fs::read_to_string(entry)?;
+            registry.register(InputKind::Workflow, contents, key)?;
         }
 
         if mode.actions()
             && entry.is_file()
             && matches!(entry.file_name(), Some("action.yml" | "action.yaml"))
         {
-            let action = Action::from_file(entry, Some(input_path))?;
-            registry.register_input(action.into())?;
+            let key = InputKey::local(entry, Some(input_path))?;
+            let contents = std::fs::read_to_string(entry)?;
+            registry.register(InputKind::Action, contents, key)?;
         }
     }
 
@@ -340,31 +350,29 @@ fn collect_from_repo_slug(
         // Performance: if we're *only* collecting workflows, then we
         // can save ourselves a full repo download and only fetch the
         // repo's workflow files.
-        for workflow in client.fetch_workflows(&slug)? {
-            registry.register_input(workflow.into())?;
-        }
+        client.fetch_workflows(&slug, registry)?;
     } else {
-        let inputs = client.fetch_audit_inputs(&slug).with_context(|| {
-            tips(
-                format!(
-                    "couldn't collect inputs from https://github.com/{owner}/{repo}",
-                    owner = slug.owner,
-                    repo = slug.repo
-                ),
-                &["confirm the repository exists and that you have access to it"],
-            )
-        })?;
+        let before = registry.len();
+        client
+            .fetch_audit_inputs(&slug, registry)
+            .with_context(|| {
+                tips(
+                    format!(
+                        "couldn't collect inputs from https://github.com/{owner}/{repo}",
+                        owner = slug.owner,
+                        repo = slug.repo
+                    ),
+                    &["confirm the repository exists and that you have access to it"],
+                )
+            })?;
+        let after = registry.len();
+        let len = after - before;
 
         tracing::info!(
             "collected {len} inputs from {owner}/{repo}",
-            len = inputs.len(),
             owner = slug.owner,
             repo = slug.repo
         );
-
-        for input in inputs {
-            registry.register_input(input)?;
-        }
     }
 
     Ok(())
@@ -374,18 +382,28 @@ fn collect_from_repo_slug(
 fn collect_inputs(
     inputs: &[String],
     mode: &CollectionMode,
+    strict: bool,
     state: &AuditState,
 ) -> Result<InputRegistry> {
-    let mut registry = InputRegistry::new();
+    let mut registry = InputRegistry::new(strict);
 
     for input in inputs {
         let input_path = Utf8Path::new(input);
         if input_path.is_file() {
             // When collecting individual files, we don't know which part
             // of the input path is the prefix.
-            registry
-                .register_by_path(input_path, None)
-                .with_context(|| format!("failed to register input: {input_path}"))?;
+            let (key, kind) = match (input_path.file_stem(), input_path.extension()) {
+                (Some("action"), Some("yml" | "yaml")) => {
+                    (InputKey::local(input_path, None)?, InputKind::Action)
+                }
+                (Some(_), Some("yml" | "yaml")) => {
+                    (InputKey::local(input_path, None)?, InputKind::Workflow)
+                }
+                _ => return Err(anyhow!("invalid input: {input}")),
+            };
+
+            let contents = std::fs::read_to_string(input_path)?;
+            registry.register(kind, contents, key)?;
         } else if input_path.is_dir() {
             collect_from_dir(input_path, mode, &mut registry)?;
             // collect_from_repo_dir(input_path, input_path, mode, &mut registry)?;
@@ -403,11 +421,26 @@ fn collect_inputs(
     Ok(registry)
 }
 
+fn completions<G: clap_complete::Generator>(generator: G, cmd: &mut clap::Command) {
+    clap_complete::generate(
+        generator,
+        cmd,
+        cmd.get_name().to_string(),
+        &mut std::io::stdout(),
+    );
+}
+
 fn run() -> Result<ExitCode> {
     human_panic::setup_panic!();
 
     let mut app = App::parse();
 
+    if let Some(shell) = app.completions {
+        let mut cmd = App::command();
+        completions(shell, &mut cmd);
+        return Ok(ExitCode::SUCCESS);
+    }
+
     let color_mode = match app.color {
         Some(color_mode) => color_mode,
         None => {
@@ -488,7 +521,12 @@ fn run() -> Result<ExitCode> {
     })?;
 
     let audit_state = AuditState::new(&app, &config);
-    let registry = collect_inputs(&app.inputs, &app.collect, &audit_state)?;
+    let registry = collect_inputs(
+        &app.inputs,
+        &app.collect,
+        app.strict_collection,
+        &audit_state,
+    )?;
 
     let mut audit_registry = AuditRegistry::new();
     macro_rules! register_audit {
@@ -512,6 +550,7 @@ fn run() -> Result<ExitCode> {
     }
 
     register_audit!(audit::artipacked::Artipacked);
+    register_audit!(audit::unsound_contains::UnsoundContains);
     register_audit!(audit::excessive_permissions::ExcessivePermissions);
     register_audit!(audit::dangerous_triggers::DangerousTriggers);
     register_audit!(audit::impostor_commit::ImpostorCommit);
@@ -530,6 +569,9 @@ fn run() -> Result<ExitCode> {
     register_audit!(audit::overprovisioned_secrets::OverprovisionedSecrets);
     register_audit!(audit::unredacted_secrets::UnredactedSecrets);
     register_audit!(audit::forbidden_uses::ForbiddenUses);
+    register_audit!(audit::obfuscation::Obfuscation);
+    register_audit!(audit::stale_action_refs::StaleActionRefs);
+    register_audit!(audit::unpinned_images::UnpinnedImages);
 
     let mut results = FindingRegistry::new(&app, &config);
     {
