@@ -1,4 +1,5 @@
 use anyhow::{Context as _, Result};
+use regex::{Captures, Regex};
 use std::{env::current_dir, io::ErrorKind};
 
 use assert_cmd::Command;
@@ -24,6 +25,7 @@ pub fn input_under_test(name: &str) -> String {
 
 pub enum OutputMode {
     Stdout,
+    #[allow(dead_code, reason = "currently not used by any integration test")]
     Stderr,
     Both,
 }
@@ -35,6 +37,7 @@ pub struct Zizmor {
     inputs: Vec<String>,
     config: Option<String>,
     output: OutputMode,
+    expects_failure: bool,
 }
 
 impl Zizmor {
@@ -49,6 +52,7 @@ impl Zizmor {
             inputs: vec![],
             config: None,
             output: OutputMode::Stdout,
+            expects_failure: false,
         }
     }
 
@@ -92,6 +96,14 @@ impl Zizmor {
         self
     }
 
+    pub fn expects_failure(mut self, flag: bool) -> Self {
+        if flag {
+            self = self.output(OutputMode::Both);
+        }
+        self.expects_failure = flag;
+        self
+    }
+
     pub fn run(mut self) -> Result<String> {
         if self.offline {
             self.cmd.arg("--offline");
@@ -149,8 +161,28 @@ impl Zizmor {
             OutputMode::Both => [output.stderr, output.stdout].concat(),
         })?;
 
+        if let Some(exit_code) = output.status.code() {
+            // There are other nonzero exit codes that don't indicate failure;
+            // 1 is our only failure code.
+            let is_failure = exit_code == 1;
+            if is_failure != self.expects_failure {
+                anyhow::bail!("zizmor exited with unexpected code {exit_code}");
+            }
+        }
+
+        let input_placeholder = "@@INPUT@@";
         for input in &self.inputs {
-            raw = raw.replace(input, "@@INPUT@@");
+            raw = raw.replace(input, input_placeholder);
+        }
+
+        // Normalize Windows '\' file paths to using '/', to get consistent snapshot test outputs
+        if cfg!(windows) {
+            let input_path_regex = Regex::new(&format!(r"{input_placeholder}[\\/\w.-]+"))?;
+            raw = input_path_regex
+                .replace_all(&raw, |captures: &Captures| {
+                    captures.get(0).unwrap().as_str().replace("\\", "/")
+                })
+                .into_owned();
         }
 
         Ok(raw)
